@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py,v 1.138 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.138 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: orca.py,v 1.141 2006/06/29 16:13:15 richb Exp $"
+__version__   = "$Revision: 1.141 $"
+__date__      = "$Date: 2006/06/29 16:13:15 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -45,6 +45,7 @@ import time
 import atspi
 import braille
 import debug
+import httpserver
 import keynames
 import keybindings
 import mag
@@ -74,6 +75,11 @@ _userSettings = None
 #
 lastInputEvent = None
 
+# The last timestamp from a device event. Used to set focus for the Orca
+# configuration GUI.
+#
+lastTimestamp = None
+
 # A new modifier to use (set by the press of any key in the
 # settings.orcaModifierKeys list) to represent the Orca modifier.
 #
@@ -688,8 +694,11 @@ def _processKeyboardEvent(event):
     Returns True if the event should be consumed.
     """
     global lastInputEvent
+    global lastTimestamp
     global _orcaModifierPressed
 
+    lastTimestamp = event.timestamp
+
     # Log the keyboard event for future playback, if desired.
     # Note here that the key event_string being output is
     # exactly what we received.  The KeyboardEvent object,
@@ -834,6 +843,8 @@ def loadUserSettings(script=None, inputEvent=None):
     global _userSettings
 
     # Shutdown the output drivers and give them a chance to die.
+    #
+    httpserver.shutdown()
     speech.shutdown()
     braille.shutdown()
     mag.shutdown()
@@ -888,6 +899,8 @@ def loadUserSettings(script=None, inputEvent=None):
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Magnification module has NOT been initialized.")
 
+    httpserver.init()
+    
     return True
 
 def _showPreferencesGUI(script=None, inputEvent=None):
@@ -1258,15 +1271,6 @@ def main():
     except:
         pass
 
-    # Do not run Orca if accessibility has not been enabled.
-    #
-    import commands
-    a11yEnabled = commands.getoutput(\
-        "gconftool-2 --get /desktop/gnome/interface/accessibility")
-    if a11yEnabled != "true":
-        _showPreferencesConsole()
-        abort()
-
     # Parse the command line options.
     #
     # Run the preferences setup if the user has specified
@@ -1313,6 +1317,15 @@ def main():
         usage()
         os._exit(2)
 
+    # Do not run Orca if accessibility has not been enabled.
+    #
+    import commands
+    a11yEnabled = commands.getoutput(\
+        "gconftool-2 --get /desktop/gnome/interface/accessibility")
+    if a11yEnabled != "true":
+        _showPreferencesConsole()
+        abort()
+
     if setupRequested and (not bypassSetup) and (not showGUI):
         _showPreferencesConsole()
 
