@@ -20,15 +20,13 @@
 """Manages the default speech server for orca.  A script can use this
 as its speech server, or it can feel free to create one of its own."""
 
-__id__        = "$Id: speech.py,v 1.55 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.55 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
+__id__        = "$Id: speech.py,v 1.56 2006/06/14 20:20:20 wwalker Exp $"
+__version__   = "$Revision: 1.56 $"
+__date__      = "$Date: 2006/06/14 20:20:20 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import threading
 import time
-import BaseHTTPServer
 
 import debug
 import orca
@@ -38,59 +36,6 @@ import settings
 from acss import ACSS
 from orca_i18n import _           # for gettext support
 
-class _SpeakRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
-    """Provides support for communicating with speech via a HTTP.
-    This is to support self-voicing applications that want to use
-    Orca as a speech service.
-
-    The protocol is simple: POST content is 'stop' or 'speak:text'
-
-    To test this, run:
-
-      wget --post-data='speak:hello world' localhost:20433
-
-    """
-
-    def do_GET(self):
-        self.send_response(200)
-        self.send_header("Content-type", "text/html")
-        self.end_headers()
-        self.wfile.write("<html><body><p>Orca %s</p></body></html>" \
-                         % platform.version)
-
-    def do_POST(self):
-        contentLength = self.headers.getheader('content-length')
-        if contentLength:
-            contentLength = int(contentLength)
-            inputBody = self.rfile.read(contentLength)
-            debug.println(debug.LEVEL_FINEST,
-                          "speech._SpeakRequestHandler received %s" \
-                          % inputBody)
-            if inputBody.startswith("speak:"):
-                speak(inputBody[6:])
-                self.send_response(200, 'OK')
-            elif inputBody == "stop":
-                stop()
-                self.send_response(200, 'OK')
-            elif inputBody == "isSpeaking":
-                self.send_response(200, 'OK')
-                self.send_header("Content-type", "text/html")
-                self.end_headers()
-                self.wfile.write("%s" % isSpeaking())
-        else:
-            debug.println(debug.LEVEL_FINEST,
-                          "speech._SpeakRequestHandler received no data")
-
-
-class _SpeakRequestThread(threading.Thread):
-    """Runs a _SpeakRequestHandler in a separate thread."""
-
-    def run(self):
-        httpd = BaseHTTPServer.HTTPServer(('',
-                                           settings.speechServerPort),
-                                          _SpeakRequestHandler)
-        httpd.serve_forever()
-
 # The speech server to use for all speech operations.
 #
 __speechserver = None
@@ -116,8 +61,6 @@ def getSpeechServerFactories():
 
     return factories
 
-_speakRequestThread = None
-
 def init():
 
     global __speechserver
@@ -159,19 +102,6 @@ def init():
     else:
         __speechserver = factory.SpeechServer.getSpeechServer()
 
-    # If we want to listen for speak commands from a separate port,
-    # do so.  We make it a daemon so it will die automatically when
-    # orca dies.
-    #
-    global _speakRequestThread
-    if settings.speechServerPort and (not _speakRequestThread):
-        try:
-            _speakRequestThread = _SpeakRequestThread()
-            _speakRequestThread.setDaemon(True)
-            _speakRequestThread.start()
-        except:
-            debug.printException(debug.LEVEL_SEVERE)
-
 def __resolveACSS(acss=None):
     if acss:
         return acss
